@@ -10,10 +10,13 @@ const { parseAddQueueTypes } = require( "./Cmd4PriorityPollingQueue" );
 
 let Logger = require( "./utils/Logger" );
 let getAccessoryUUID = require( "./utils/getAccessoryUUID" );
-let ucFirst = require( "./utils/ucFirst" );
+let lcFirst = require( "./utils/lcFirst" );
 let isNumeric = require( "./utils/isNumeric" );
 let trueTypeOf = require( "./utils/trueTypeOf" );
 
+// Hierarchy variables
+let HV = require( "./utils/HV" );
+
 let createAccessorysInformationService = require( "./utils/createAccessorysInformationService" );
 
 // Pretty Colors
@@ -34,7 +37,6 @@ const Cmd4Accessory = require( "./Cmd4Accessory" ).Cmd4Accessory;
 // Settings, Globals and Constants
 let settings = require( "./cmd4Settings" );
 const constants = require( "./cmd4Constants" );
-let cmd4Dbg = settings.cmd4Dbg;
 
 // Platform definition
 class Cmd4Platform
@@ -48,7 +50,7 @@ class Cmd4Platform
          this.log = log;
 
          // Carry the debug flag from the platform
-         cmd4Dbg = log.debugEnabled;
+         settings.cmd4Dbg = log.debugEnabled;
       }
       else
       {
@@ -56,17 +58,16 @@ class Cmd4Platform
          this.log = new Logger( );
 
          if ( config[ constants.DEBUG ]  == true ||
-              config[ constants.DEBUG_l ]  == true ||
+              config[ "Debug" ]  == true ||
               process.env.DEBUG == settings.PLATFORM_NAME )
          {
             settings.cmd4Dbg = true;
-            cmd4Dbg = true;
          }
       }
 
-      this.log.setDebugEnabled( cmd4Dbg );
+      this.log.setDebugEnabled( settings.cmd4Dbg );
 
-      if ( cmd4Dbg ) this.log.debug( chalk.blue( `Class Cmd4Platform` ) );
+      if ( settings.cmd4Dbg ) this.log.debug( chalk.blue( `Class Cmd4Platform` ) );
 
       if ( config === undefined )
          return;
@@ -94,16 +95,19 @@ class Cmd4Platform
       // These would be queues of Characteristics to be polled or get/set via IOS.
       settings.listOfCreatedPriorityQueues = { };
 
-
-      // Defaults before parse
-      this.outputConstants = constants.DEFAULT_OUTPUTCONSTANTS;
-      this.statusMsg = constants.DEFAULT_STATUSMSG;
-
       // Track the polling timers only so that unit testing can cancel them.
       this.pollingTimers = [ ];
 
+      // Create the hierarhy variables
+      this.hV = new HV();
+
       this.parseConfigForCmd4Directives( this.config );
 
+      // Update the namespace for stored variables
+      // like timeout, stateChangeResponseTime ... As it may require
+      // changes from parseConfig.
+      this.hV.update( this );
+
       this.processNewCharacteristicDefinitions( );
 
 
@@ -145,7 +149,7 @@ class Cmd4Platform
    {
       if ( platformAccessory )
       {
-         if ( cmd4Dbg ) this.log.debug( `Found cached accessory: ${ platformAccessory.displayName }` );
+         if ( settings.cmd4Dbg ) this.log.debug( `Found cached accessory: ${ platformAccessory.displayName }` );
          this.toBeRestoredPlatforms.push( platformAccessory );
       }
    }
@@ -178,14 +182,22 @@ class Cmd4Platform
    {
       for ( let key in config )
       {
+         let lcKey = lcFirst( key );
+         if ( key == "UUID" )
+         {
+            lcKey = "uuid";
+         } else
+         {
+            // warn now
+            if ( key.charAt( 0 ) === key.charAt( 0 ).toUpperCase( ) )
+            {
+               this.log.warn( `The config.json Platform key: ${ key } is Capitalized.  All keys in the near future will ALWAYS start with a lower case character for homebridge-ui integration.\nTo remove this Warning, Please fix your config.json.` );
+            }
+         }
+
          let value = config[ key ];
 
-         // I made the stupid mistake of not having all characteristics in
-         // the config.json file not upper case to match that in State.js.
-         // So instead of having everyone fix their scripts, fix it here.
-         let ucKey = ucFirst( key );
-
-         switch ( ucKey )
+         switch ( lcKey )
          {
             case constants.TIMEOUT:
                // Timers are in milliseconds. A low value can result in
@@ -235,12 +247,6 @@ class Cmd4Platform
 
                break;
             case "QueueStatMsgInterval":
-
-               // Never put into production
-               this.log.warn( `Warning: ${ key } has been deprecated. It was never even used.` );
-               this.log.warn( `To remove this message, just remove ${ key } from your config.json` );
-
-               break;
             case "QueueMsg":
 
                // Never put into production
@@ -253,6 +259,7 @@ class Cmd4Platform
 
                break;
             case "Cmd4_Mode":
+            case "cmd4_Mode":
                this.log.warn( `Warning: ${ key } has been deprecated.` );
                if ( value == "Demo" )
                {
@@ -264,10 +271,10 @@ class Cmd4Platform
                this.log.warn( `To remove this message, just remove "${ key }" from your config.json` );
                break;
             case constants.STORAGE:
-               if ( value == constants.FS_l || value == constants.GOOGLEDRIVE_l )
+               if ( value == constants.FS || value == constants.GOOGLEDRIVEl )
                   this.storage = value;
                else
-                  this.log.warn( chalk.yellow( `WARNING` ) + `: Cmd4 Unknown platform.config.storage:{ this.storage } Expected:${ constants.FS_l } or ${ constants.GOOGLEDRIVE_l }` );
+                  this.log.warn( chalk.yellow( `WARNING` ) + `: Cmd4 Unknown platform.config.storage:{ this.storage } Expected:${ constants.FS } or ${ constants.GOOGLEDRIVE }` );
 
                break;
             case constants.STORAGEPATH:
@@ -315,7 +322,7 @@ class Cmd4Platform
 
       this.definitions.forEach( ( definition, definitionIndex ) =>
       {
-         if ( cmd4Dbg ) this.log.debug( `Processing definition index: ${ definitionIndex }` );
+         if ( settings.cmd4Dbg ) this.log.debug( `Processing definition index: ${ definitionIndex }` );
 
          if ( trueTypeOf( definition.type ) != String )
             throw new Error( `definition.type at index: ${ definitionIndex } is not a String.` );
@@ -383,7 +390,7 @@ class Cmd4Platform
          }
 
          CMD4_ACC_TYPE_ENUM.add( this.api, definition.type, definition.description, definition.props, definition.validValues );
-         if ( cmd4Dbg ) this.log.debug( `Created definition type: "${ definition.type }".` );
+         if ( settings.cmd4Dbg ) this.log.debug( `Created definition type: "${ definition.type }".` );
 
       });
    }
@@ -398,7 +405,7 @@ class Cmd4Platform
       // already been registered.
       this.config.accessories && this.config.accessories.forEach( ( device ) =>
       {
-         if ( cmd4Dbg ) this.log.debug( `Fetching config.json Platform accessories.` );
+         if ( settings.cmd4Dbg ) this.log.debug( `Fetching config.json Platform accessories.` );
          this.Service=this.api.hap.Service;
 
          device.name = getAccessoryName( device );
@@ -407,19 +414,24 @@ class Cmd4Platform
          // generate a unique id for the accessory this should be generated from
          // something globally unique, but constant, for example, the device serial
          // number or MAC address.
-         let UUID = getAccessoryUUID( device, this.api.hap.uuid );
+         let uuid = getAccessoryUUID( device, this.api.hap.uuid );
 
          // See if an accessory with the same UUID has already been registered and
          // restored from the cached devices we stored in the `configureAccessory`
          // method above
-         const existingAccessory = this.toBeRestoredPlatforms.find(accessory => accessory.UUID === UUID);
+
+         // NOTE: HOMEBRIDGE EXAMPLES HAVE THIS AS UPPERCASE UUID.
+         //       lower case uuid will not be found.
+         const existingAccessory = this.toBeRestoredPlatforms.find(accessory => accessory.UUID === uuid);
 
          if (existingAccessory)
          {
+            // NOTE: HOMEBRIDGE EXAMPLES HAVE THIS AS UPPERCASE UUID.
+            //       lower case uuid will not be found.
             let duplicatePlatformAccessory = this.createdCmd4Platforms.find(accessory => accessory.UUID === existingAccessory.UUID);
             if ( duplicatePlatformAccessory )
             {
-               this.log( chalk.red( `Error duplicate platform accessory: ${ duplicatePlatformAccessory.name } UUID:${ duplicatePlatformAccessory.UUID }` ) );
+               this.log( chalk.red( `Error duplicate platform accessory: ${ duplicatePlatformAccessory.name } uuid:${ duplicatePlatformAccessory.UUID }` ) );
                // Next in for.Each object iteration
                return;
             }
@@ -441,8 +453,13 @@ class Cmd4Platform
             platform.Service = this.Service;
 
             // This is how we keep the device status information over restart.
+            // Version 0
             // Within STORED_DATA_ARRAY is a list of UUID identified objects
             // of storedValuesPerCharacteristic. 
+            // Version 1
+            // Within STORED_DATA_ARRAY is a list of UUID identified objects
+            // of Cmd4Storage.
+
             // If the accessory has linked accessories or standalone accessories,
             // there infomation gets put in this as well.  I'm pretty sure
             // that previosly these states were lost.
@@ -454,18 +471,9 @@ class Cmd4Platform
             // If the saved context has our STORED_DATA_ARRAY, then use it.
             if ( existingAccessory.context.STORED_DATA_ARRAY )
             {
-               if ( cmd4Dbg ) this.log.debug(`Cmd4Platform: Using context.STORED_DATA_ARRAY` );
+               if ( settings.cmd4Dbg ) this.log.debug(`Cmd4Platform: Using context.STORED_DATA_ARRAY` );
                STORED_DATA_ARRAY = existingAccessory.context.STORED_DATA_ARRAY;
-            } else if ( existingAccessory.context.device.storedValuesPerCharacteristic )
-            {
-               // If we have an old version of the stored status, convert it to our new format.
-               if ( cmd4Dbg ) this.log.debug(`Cmd4Platform: Creating STORED_DATA_ARRAY with existing storedValuesPerCharacteristic UUID:${ existingAccessory.UUID }` );
-               STORED_DATA_ARRAY = [ {[ constants.UUID ]: existingAccessory.UUID,
-                                      [ constants.STORED_VALUES_PER_CHARACTERISTIC_lv ]: existingAccessory.context.device.storedValuesPerCharacteristic }
-                      ];
-               // Get rid of old persistance data
-               existingAccessory.context.device = undefined;
-               delete( existingAccessory.context.device );
+
             }
 
             let that = this;
@@ -483,7 +491,7 @@ class Cmd4Platform
             // Get the properties for this accessories device type
             let devProperties = CMD4_DEVICE_TYPE_ENUM.properties[ accessory.typeIndex ];
 
-            if ( cmd4Dbg ) this.log.debug( `Step 2. ${ accessory.displayName }.service = platform.getService( Service.${ devProperties.deviceName }, ${ accessory.subType })` );
+            if ( settings.cmd4Dbg ) this.log.debug( `Step 2. ${ accessory.displayName }.service = platform.getService( Service.${ devProperties.deviceName }, ${ accessory.subType })` );
             accessory.service = platform.getService( devProperties.service, accessory.name, accessory.subType );
 
             // Determine which characteristics, if any, will be polled. This
@@ -511,8 +519,8 @@ class Cmd4Platform
             // Create the new PlatformAccessory
             if ( device.category == undefined )
             {
-               if ( cmd4Dbg ) this.log.debug( `Step 1. platformAccessory = new platformAccessory( ${ displayName }, ${ UUID } )` );
-               platform = new this.api.platformAccessory( displayName, UUID );
+               if ( settings.cmd4Dbg ) this.log.debug( `Step 1. platformAccessory = new platformAccessory( ${ displayName }, ${ uuid } )` );
+               platform = new this.api.platformAccessory( displayName, uuid );
 
             } else
             {
@@ -523,9 +531,9 @@ class Cmd4Platform
                if ( ! category )
                   throw new Error( `Category specified: ${ device.category } is not a valid homebridge category.` );
 
-               if ( cmd4Dbg ) this.log.debug( `Step 1. platformAccessory = new platformAccessory( ${ displayName }, ${ UUID }, ${ category } )` );
+               if ( settings.cmd4Dbg ) this.log.debug( `Step 1. platformAccessory = new platformAccessory( ${ displayName }, ${ uuid }, ${ category } )` );
 
-               platform = new this.api.platformAccessory( displayName, UUID, category );
+               platform = new this.api.platformAccessory( displayName, uuid, category );
             }
 
             platform.Service = this.Service;
@@ -538,7 +546,7 @@ class Cmd4Platform
             // Put the accessory into its correct collection array.
             this.createdCmd4Accessories.push( accessory );
 
-            if ( cmd4Dbg ) this.log.debug( `Created platformAccessory: ${ accessory.displayName }` );
+            if ( settings.cmd4Dbg ) this.log.debug( `Created platformAccessory: ${ accessory.displayName }` );
 
             // Store a copy of the device object in the `accessory.context`
             // the `context` property can be used to store any data about the
@@ -550,7 +558,7 @@ class Cmd4Platform
 
             // MOVE OUSTSIDE
             // Platform Step 2. const tvService = this.tvAccessory.addService( this.Service.Television );
-            if ( cmd4Dbg ) this.log.debug( `Step 2. ${ accessory.displayName }.service = platform.addService( this.Service.${ devProperties.deviceName }, ${ accessory.name }, ${ accessory.subType })` );
+            if ( settings.cmd4Dbg ) this.log.debug( `Step 2. ${ accessory.displayName }.service = platform.addService( this.Service.${ devProperties.deviceName }, ${ accessory.name }, ${ accessory.subType })` );
             accessory.service = platform.addService( new devProperties.service( accessory.name, accessory.subType )  );
 
             // Create all the services for the accessory, including fakegato
@@ -560,12 +568,12 @@ class Cmd4Platform
             // Step 6. this.api.publishExternalAccessories( PLUGIN_NAME, [ this.tvAccessory ] );
             if ( accessory.publishExternally )
             {
-               if ( cmd4Dbg ) this.log.debug( `Step 6. publishExternalAccessories( ${ settings.PLUGIN_NAME }, [ ${accessory.displayName } ] )` );
+               if ( settings.cmd4Dbg ) this.log.debug( `Step 6. publishExternalAccessories( ${ settings.PLUGIN_NAME }, [ ${accessory.displayName } ] )` );
 
                this.api.publishExternalAccessories( settings.PLUGIN_NAME, [ platform ] );
 
             } else {
-               if ( cmd4Dbg ) this.log.debug( `Step 6. registerPlatformAccessories( ${ settings.PLUGIN_NAME }, ${ settings.PLATFORM_NAME }, [ ${  accessory.displayName } ] ) `);
+               if ( settings.cmd4Dbg ) this.log.debug( `Step 6. registerPlatformAccessories( ${ settings.PLUGIN_NAME }, ${ settings.PLATFORM_NAME }, [ ${  accessory.displayName } ] ) `);
 
                this.api.registerPlatformAccessories( settings.PLUGIN_NAME, settings.PLATFORM_NAME, [ platform ] );
             }
@@ -589,28 +597,28 @@ class Cmd4Platform
       // for us and the getService hangs off the platform, not the accessory.
       if ( cmd4PlatformAccessory.model )
       {
-         if ( cmd4Dbg ) cmd4PlatformAccessory.log.debug( `Adding model( ${ cmd4PlatformAccessory.model } ) to information service of ${ cmd4PlatformAccessory.displayName }` );
+         if ( settings.cmd4Dbg ) cmd4PlatformAccessory.log.debug( `Adding model( ${ cmd4PlatformAccessory.model } ) to information service of ${ cmd4PlatformAccessory.displayName }` );
          cmd4PlatformAccessory.platform.getService( cmd4PlatformAccessory.platform.Service.AccessoryInformation )
             .setCharacteristic( this.api.hap.Characteristic.Model, cmd4PlatformAccessory.model );
       }
 
       if ( cmd4PlatformAccessory.manufacturer )
       {
-         if ( cmd4Dbg ) cmd4PlatformAccessory.log.debug( `Adding manufacturer( ${ cmd4PlatformAccessory.manufacturer } ) to information service of ${ cmd4PlatformAccessory.displayName }` );
+         if ( settings.cmd4Dbg ) cmd4PlatformAccessory.log.debug( `Adding manufacturer( ${ cmd4PlatformAccessory.manufacturer } ) to information service of ${ cmd4PlatformAccessory.displayName }` );
          cmd4PlatformAccessory.platform.getService( cmd4PlatformAccessory.platform.Service.AccessoryInformation )
             .setCharacteristic( this.api.hap.Characteristic.Manufacturer, cmd4PlatformAccessory.manufacturer );
       }
 
       if ( cmd4PlatformAccessory.serialNumber )
       {
-         if ( cmd4Dbg ) cmd4PlatformAccessory.log.debug( `Adding serial Number( ${ cmd4PlatformAccessory.serialNumber } ) to information service of ${ cmd4PlatformAccessory.displayName }` );
+         if ( settings.cmd4Dbg ) cmd4PlatformAccessory.log.debug( `Adding serial Number( ${ cmd4PlatformAccessory.serialNumber } ) to information service of ${ cmd4PlatformAccessory.displayName }` );
          cmd4PlatformAccessory.platform.getService( cmd4PlatformAccessory.platform.Service.AccessoryInformation )
             .setCharacteristic( this.api.hap.Characteristic.SerialNumber, cmd4PlatformAccessory.serialNumber );
       }
 
       if ( cmd4PlatformAccessory.firmwareRevision )
       {
-         if ( cmd4Dbg ) cmd4PlatformAccessory.log.debug( `Adding Firmware Revision( ${ cmd4PlatformAccessory.firmwareRevision } ) to information service of ${ cmd4PlatformAccessory.displayName }` );
+         if ( settings.cmd4Dbg ) cmd4PlatformAccessory.log.debug( `Adding Firmware Revision( ${ cmd4PlatformAccessory.firmwareRevision } ) to information service of ${ cmd4PlatformAccessory.displayName }` );
          cmd4PlatformAccessory.platform.getService( cmd4PlatformAccessory.platform.Service.AccessoryInformation )
             .setCharacteristic( this.api.hap.Characteristic.FirmwareRevision, cmd4PlatformAccessory.firmwareRevision );
       }
@@ -630,19 +638,19 @@ class Cmd4Platform
          // Existing Accessories would have existing services
          if ( fromExisting == true )
          {
-            if ( cmd4Dbg ) this.log.debug( `Platform (AddedAccessory-existing) Step 3, ${ addedAccessory.displayName }.service = accessory.platform.getService( Service.${ devProperties.deviceName }, ${ addedAccessory.name }, ${ addedAccessory.subType }` );
+            if ( settings.cmd4Dbg ) this.log.debug( `Platform (AddedAccessory-existing) Step 3, ${ addedAccessory.displayName }.service = accessory.platform.getService( Service.${ devProperties.deviceName }, ${ addedAccessory.name }, ${ addedAccessory.subType }` );
             addedAccessory.service = addedAccessory.platform.getService( devProperties.service, addedAccessory.name, addedAccessory.subType );
          } else
          {
 
-            if ( cmd4Dbg ) this.log.debug( `Platform (AddedAccessory-new) Step 3, ${ addedAccessory.displayName }.service = PlatformAccessory: ${ cmd4PlatformAccessory.displayName } addService( Service:${ devProperties.deviceName }, ${ addedAccessory.name }, ${ addedAccessory.subType } )` );
+            if ( settings.cmd4Dbg ) this.log.debug( `Platform (AddedAccessory-new) Step 3, ${ addedAccessory.displayName }.service = PlatformAccessory: ${ cmd4PlatformAccessory.displayName } addService( Service:${ devProperties.deviceName }, ${ addedAccessory.name }, ${ addedAccessory.subType } )` );
 
             addedAccessory.service = cmd4PlatformAccessory.platform.addService( new devProperties.service( addedAccessory.name, addedAccessory.subType ) );
          }
 
          addedAccessory.addAllServiceCharacteristicsForAccessory( addedAccessory );
          // Create Information Service for the addedAccessory
-         if ( cmd4Dbg ) addedAccessory.log.debug( `Creating information service for AddedAccessory: ${ addedAccessory.displayName }` );
+         if ( settings.cmd4Dbg ) addedAccessory.log.debug( `Creating information service for AddedAccessory: ${ addedAccessory.displayName }` );
          createAccessorysInformationService( addedAccessory );
 
           // Setup the fakegato service if defined in the config.json file
@@ -669,12 +677,12 @@ class Cmd4Platform
          // Existing Accessories would have existing services
          if ( fromExisting == true )
          {
-            if ( cmd4Dbg ) this.log.debug( `Platform (LinkedAccessory-existing) Step 4. ${ linkedAccessory.displayName }.service = ${ cmd4PlatformAccessory.displayName }.getService:( ${ devProperties.deviceName }.service, ${linkedAccessory.name }, ${linkedAccessory.subType } )` );
+            if ( settings.cmd4Dbg ) this.log.debug( `Platform (LinkedAccessory-existing) Step 4. ${ linkedAccessory.displayName }.service = ${ cmd4PlatformAccessory.displayName }.getService:( ${ devProperties.deviceName }.service, ${linkedAccessory.name }, ${linkedAccessory.subType } )` );
 
             linkedAccessory.service = linkedAccessory.platform.getService( devProperties.service, linkedAccessory.name, linkedAccessory.subType );
          } else
          {
-            if ( cmd4Dbg ) this.log.debug( `Platform (LinkedAccessory-new) Step 4. ${ linkedAccessory.displayName }.service = ${ cmd4PlatformAccessory.displayName }.addService:( ${ devProperties.deviceName }.service, ${linkedAccessory.name }, ${linkedAccessory.subType } )` );
+            if ( settings.cmd4Dbg ) this.log.debug( `Platform (LinkedAccessory-new) Step 4. ${ linkedAccessory.displayName }.service = ${ cmd4PlatformAccessory.displayName }.addService:( ${ devProperties.deviceName }.service, ${linkedAccessory.name }, ${linkedAccessory.subType } )` );
 
             linkedAccessory.service = cmd4PlatformAccessory.platform.addService( new devProperties.service( linkedAccessory.name, linkedAccessory.subType ) );
          }
@@ -683,12 +691,12 @@ class Cmd4Platform
 
          if ( fromExisting == false )
          {
-            if ( cmd4Dbg ) this.log.debug( `Platform Step 5. ${ cmd4PlatformAccessory.displayName }.service.addLinkedService( ${ linkedAccessory.displayName }.service )` );
+            if ( settings.cmd4Dbg ) this.log.debug( `Platform Step 5. ${ cmd4PlatformAccessory.displayName }.service.addLinkedService( ${ linkedAccessory.displayName }.service )` );
             cmd4PlatformAccessory.service.addLinkedService( linkedAccessory.service );
          }
 
          // Create Information Service for the linkedAccessory
-         if ( cmd4Dbg ) linkedAccessory.log.debug( `Creating information service for Linked Platform Accessory: ${ linkedAccessory.displayName }` );
+         if ( settings.cmd4Dbg ) linkedAccessory.log.debug( `Creating information service for Linked Platform Accessory: ${ linkedAccessory.displayName }` );
          createAccessorysInformationService( linkedAccessory );
 
           // Setup the fakegato service if defined in the config.json file
@@ -715,7 +723,7 @@ class Cmd4Platform
       // Check for any queued characteristics
       if ( Object.keys( settings.listOfCreatedPriorityQueues ).length == 0 )
       {
-         if ( cmd4Dbg ) this.log.debug( `No queued polling characteristics` );
+         if ( settings.cmd4Dbg ) this.log.debug( `No queued polling characteristics` );
          return;
       }
 
